USE AdventureWorks2014;
SET NOCOUNT ON;
GO

SELECT SUM(TotalDue)
FROM Sales.SalesOrderHeader
WHERE (OrderDate >= '20050701'
   AND OrderDate <  '20050705')
   OR (OrderDate >= '20060101'
   AND OrderDate <  '20060105');

-- we hit a tipping point here 
SELECT SUM(TotalDue) 
FROM Sales.SalesOrderHeader
WHERE (OrderDate >= '20050701'
   AND OrderDate <  '20050801') -- Just a little more data than Q1
   OR (OrderDate >= '20060101'
   AND OrderDate <  '20060105');
      
SELECT SUM(TotalDue) 
FROM Sales.SalesOrderHeader --WITH (FORCESEEK)
WHERE (OrderDate >= '20050701'
   AND OrderDate <  '20050801')
   OR (OrderDate >= '20060101'
   AND OrderDate <  '20060105');


sp_helpindex 'Sales.SalesOrderHeader'
GO

-- The column SalesPersonID is the leading key in index IX_SalesOrderHeader_SalesPersonID.
-- Which of the below queries, all of which filter on SalesPersonID, are SARGable,
-- meaning that they will use that index to seek out matching rows?

-- SARGable?  Yes
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE SalesPersonID = 283

-- SARGable?  Depends on SQL version (2008+) and available statistics
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE SalesPersonID <> 283
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE SalesPersonID > 283 OR SalesPersonID < 283

-- SARGable?  Yes
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE SalesPersonID >= 283

-- SARGable?  No
SELECT SalesOrderID FROM Sales.SalesOrderHeader 
	WHERE SalesPersonID NOT IN 
		(SELECT BusinessEntityID FROM Person.Person WHERE LastName = 'Campbell' AND FirstName = 'David')

-- SARGable?  Yes
SELECT SalesOrderID FROM Sales.SalesOrderHeader 
	WHERE SalesPersonID IN 
		(SELECT BusinessEntityID FROM Person.Person WHERE LastName = 'Campbell' AND FirstName = 'David')

-- SARGable?  No
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE ISNULL(SalesPersonID, 0) = 283

-- SARGable?  Yes
SELECT SalesOrderID FROM Sales.SalesOrderHeader WHERE SalesPersonID IS NULL

SELECT SOH.CustomerID
	,SOH.OrderDate
	,SOH.STATUS
	,SOH.SalesOrderID
	FROM Sales.SalesOrderHeader SOH
	INNER JOIN 
		(SELECT CustomerID 
		,MAX (OrderDate) as ODate
		FROM Sales.SalesOrderHeader
		GROUP BY CustomerID) MaxOrdList
	ON MaxOrdList.ODate = SOH.OrderDate
	AND MaxOrdList.CustomerID = SOH.CustomerID
WHERE SOH.CustomerID IN (11276, 11201);


--New Solution.  Constant cost regardless of data set size
WITH LatestOrder AS 
(SELECT 
	SalesOrderID, RevisionNumber, OrderDate, DueDate, ShipDate, Status, OnlineOrderFlag, SalesOrderNumber, PurchaseOrderNumber, AccountNumber, CustomerID, SalesPersonID, TerritoryID, BillToAddressID, ShipToAddressID, ShipMethodID, CreditCardID, CreditCardApprovalCode, CurrencyRateID, SubTotal, TaxAmt, Freight, TotalDue, Comment, rowguid, ModifiedDate
	,ROW_NUMBER() OVER (PARTITION BY CustomerID ORDER BY OrderDate DESC )RowNum
	FROM Sales.SalesOrderHeader
	)
SELECT CustomerID
	,OrderDate
	,Status
	,SalesOrderID
	--, *
	 FROM LatestOrder
WHERE CustomerID IN (11276, 11201)
AND RowNum = 1;