-- SQL Server performance counters.
-- Capture the first counter set
SELECT  CAST(1 AS INT) AS collection_instance ,
        object_name ,
        counter_name ,
        instance_name ,
        cntr_value ,
        cntr_type ,
        CURRENT_TIMESTAMP AS collection_time
FROM    sys.dm_os_performance_counters
WHERE   ( object_name = 'SQLServer:SQL Statistics'
             AND counter_name = 'SQL Re-Compilations/sec'
           )
         OR ( object_name = 'SQLServer:SQL Statistics'
             AND counter_name = 'SQL Compilations/sec'
           );
GO 10



-- Plan Cache Examples
DBCC FREEPROCCACHE
DBCC DROPCLEANBUFFERS
GO



--up to 10 Seconds to run.
EXEC dbo.uspGetEmployeeManagers 1;
GO 99

--Returns a row for each query plan that is cached by SQL Server for faster query execution
SELECT * FROM sys.dm_exec_cached_plans 
WHERE cacheobjtype = 'Compiled Plan'
AND objtype = 'Proc'
ORDER BY usecounts DESC

--Returns one row per plan attribute for the plan specified by the plan handle.
DECLARE @plan_handle VARBINARY(64) = (SELECT TOP 1 plan_handle
	FROM sys.dm_exec_cached_plans
	WHERE cacheobjtype = 'Compiled Plan'
	AND objtype = 'Proc'
	ORDER BY usecounts DESC)

SELECT * FROM sys.dm_exec_plan_attributes (@plan_handle)
GO

--Returns the Showplan in XML format for the batch specified by the plan handle
DECLARE @plan_handle VARBINARY(64) = (SELECT TOP 1 plan_handle
	FROM sys.dm_exec_cached_plans
	WHERE cacheobjtype = 'Compiled Plan'
	AND objtype = 'Proc'
	ORDER BY usecounts DESC)

SELECT * FROM sys.dm_exec_query_plan(@plan_handle)
GO

--Returns the Showplan in text format for a Transact-SQL batch or for a specific statement within the batch.
DECLARE @plan_handle VARBINARY(64) = (SELECT TOP 1 plan_handle
	FROM sys.dm_exec_cached_plans
	WHERE cacheobjtype = 'Compiled Plan'
	AND objtype = 'Proc'
	ORDER BY usecounts DESC)
	
SELECT * FROM sys.dm_exec_text_query_plan (@plan_handle,0,-1)
GO

--TIP: sp_configure 'optimize for ad hoc workloads'
--http://www.sqlskills.com/blogs/kimberly/post/procedure-cache-and-optimizing-for-adhoc-workloads.aspx
SELECT objtype AS [CacheType]
        , count_big(*) AS [Total Plans]
        , sum(cast(size_in_bytes as decimal(18,2)))/1024/1024 AS [Total MBs]
        , avg(usecounts) AS [Avg Use Count]
        , sum(cast((CASE WHEN usecounts = 1 THEN size_in_bytes ELSE 0 END) as decimal(18,2)))/1024/1024 AS [Total MBs - USE Count 1]
        , sum(CASE WHEN usecounts = 1 THEN 1 ELSE 0 END) AS [Total Plans - USE Count 1]
FROM sys.dm_exec_cached_plans
GROUP BY objtype
ORDER BY [Total MBs - USE Count 1] DESC
GO

-- SQL Server execution statistics.
SELECT TOP 10
        execution_count ,
        statement_start_offset AS stmt_start_offset ,
        sql_handle ,
        plan_handle ,
        total_logical_reads / execution_count AS avg_logical_reads ,
        total_logical_writes / execution_count AS avg_logical_writes ,
        total_physical_reads / execution_count AS avg_physical_reads ,
        t.text
FROM    sys.dm_exec_query_stats AS s
        CROSS APPLY sys.dm_exec_sql_text(s.sql_handle) AS t
ORDER BY execution_count DESC
GO