DBCC FREEPROCCACHE;
DBCC DROPCLEANBUFFERS;
GO

CREATE PROCEDURE dbo.p01_all_default
AS
BEGIN
  DECLARE @id INT;

  DECLARE c CURSOR FOR
    SELECT TOP (1000) SalesOrderID
	  FROM Sales.SalesOrderHeader
	  ORDER BY SalesOrderID;

  OPEN c;  FETCH c INTO @id;

  WHILE @@FETCH_STATUS = 0
  BEGIN
    UPDATE Sales.SalesOrderHeader SET TaxAmt += 0
	  WHERE SalesOrderID = @id;

    FETCH c INTO @id;
  END

  CLOSE c; DEALLOCATE c;
END
GO

CREATE PROCEDURE dbo.p02_just_fast_forward
AS
BEGIN
  DECLARE @id INT;

  DECLARE c CURSOR LOCAL FAST_FORWARD FOR
    SELECT TOP (1000) SalesOrderID
	  FROM Sales.SalesOrderHeader
	  ORDER BY SalesOrderID;

  OPEN c;  FETCH c INTO @id;

  WHILE @@FETCH_STATUS = 0
  BEGIN
    UPDATE Sales.SalesOrderHeader SET TaxAmt += 0
	  WHERE SalesOrderID = @id;

    FETCH c INTO @id;
  END

  CLOSE c; DEALLOCATE c;
END
GO

CREATE PROCEDURE dbo.p03_just_nocount
AS
BEGIN
  SET NOCOUNT ON;

  DECLARE @id INT;

  DECLARE c CURSOR FOR
    SELECT TOP (1000) SalesOrderID
	  FROM Sales.SalesOrderHeader
	  ORDER BY SalesOrderID;

  OPEN c;  FETCH c INTO @id;

  WHILE @@FETCH_STATUS = 0
  BEGIN
    UPDATE Sales.SalesOrderHeader SET TaxAmt += 0
	  WHERE SalesOrderID = @id;

    FETCH c INTO @id;
  END

  CLOSE c; DEALLOCATE c;
END
GO

CREATE PROCEDURE dbo.p04__both_nocount_and_fast_forward
AS
BEGIN
  SET NOCOUNT ON;

  DECLARE @id INT;

  DECLARE c CURSOR LOCAL FAST_FORWARD FOR
    SELECT TOP (1000) SalesOrderID
	  FROM Sales.SalesOrderHeader
	  ORDER BY SalesOrderID;

  OPEN c;  FETCH c INTO @id;

  WHILE @@FETCH_STATUS = 0
  BEGIN
    UPDATE Sales.SalesOrderHeader SET TaxAmt += 0
	  WHERE SalesOrderID = @id;

    FETCH c INTO @id;
  END

  CLOSE c; DEALLOCATE c;
END
GO

EXEC dbo.p01_all_default;
EXEC dbo.p02_just_fast_forward;
EXEC dbo.p03_just_nocount;
EXEC dbo.p04__both_nocount_and_fast_forward;
GO 10

SELECT p.name, s.execution_count, s.max_elapsed_time, s.total_elapsed_time
FROM sys.dm_exec_procedure_stats AS s
INNER JOIN sys.procedures AS p
ON s.[object_id] = p.[object_id]
WHERE s.database_id = DB_ID()
AND p.name LIKE N'p0[1-4][_]%'
ORDER BY p.name;
GO

DROP PROCEDURE dbo.p01_all_default, dbo.p02_just_fast_forward, dbo.p04__both_nocount_and_fast_forward, dbo.p03_just_nocount;
GO

-- Cleanup SalesOrderHeader flags 
UPDATE Sales.SalesOrderHeader SET RevisionNumber = 0;
