USE AdventureWorks2012;
SET NOCOUNT ON;
GO

-- create two users with different default schemas
CREATE USER Orlando1 WITHOUT LOGIN 
  WITH DEFAULT_SCHEMA = Sales;
GO
CREATE USER Orlando2 WITHOUT LOGIN 
  WITH DEFAULT_SCHEMA = Person;
GO

-- give them access to an empty table
IF OBJECT_ID('dbo.SomeTable') IS NOT NULL
BEGIN
  DROP TABLE dbo.SomeTable;
END
GO
CREATE TABLE dbo.SomeTable(id INT);
GO
GRANT SELECT ON dbo.SomeTable TO Orlando1, Orlando2;
GO

-- run this batch once without schema prefix,
-- then again with the prefix (dbo) added.

DBCC FREEPROCCACHE;
DBCC FREESYSTEMCACHE('ALL');
GO

EXECUTE AS User = N'Orlando1';
GO
SELECT id FROM SomeTable;
GO
--SELECT id FROM dbo.SomeTable;
GO
REVERT;
GO
EXECUTE AS User = N'Orlando2';
GO
SELECT id FROM SomeTable;
GO
--SELECT id FROM dbo.SomeTable;
GO
REVERT;
GO

-- see how many copies of the plan exist
-- and how they are different
SELECT t.[text], p.size_in_bytes, p.usecounts, pa.value, 
  [schema] = SCHEMA_NAME(CONVERT(INT, pa.[value]))
FROM sys.dm_exec_cached_plans AS p
CROSS APPLY sys.dm_exec_sql_text(p.plan_handle) AS t
CROSS APPLY sys.dm_exec_plan_attributes(p.plan_handle) AS pa
WHERE t.[text] LIKE N'%Some'+N'Table%' 
  AND pa.attribute = N'user_id';

GO

-- clean up users & table
DROP TABLE dbo.SomeTable;
DROP USER Orlando1;
DROP USER Orlando2;