USE AdventureWorks2014;
SET NOCOUNT ON;
GO

-- create four tables to hold our data:
CREATE TABLE dbo.Numbers_Loop(Number INT PRIMARY KEY);
CREATE TABLE dbo.Numbers_CTE (Number INT PRIMARY KEY);
CREATE TABLE dbo.Numbers_Set (Number INT PRIMARY KEY);
CREATE TABLE dbo.Numbers_Num (Number INT PRIMARY KEY);
GO

-- variables to track timing:
DECLARE @d DATETIME2, @Loop INT, @CTE INT, @Set INT, @Num INT;

-- loop:

SET @d = SYSDATETIME();

DECLARE @i INT = 1;
WHILE @i <= 50000
BEGIN
  INSERT dbo.Numbers_Loop WITH (TABLOCKX) (Number)
    SELECT @i;

  SET @i += 1;
END

SELECT @Loop = DATEDIFF(MILLISECOND, @d, SYSDATETIME());

-- recursive CTE:

SET @d = SYSDATETIME();

;WITH cte AS
(
  SELECT Number = 1
  UNION ALL
  SELECT Number + 1 FROM cte
  WHERE Number <= 50000
)
INSERT dbo.Numbers_CTE WITH (TABLOCKX) (Number)
SELECT Number FROM cte
OPTION (MAXRECURSION 0);

SELECT @CTE = DATEDIFF(MILLISECOND, @d, SYSDATETIME());

-- set-based, using catalog views

SET @d = SYSDATETIME();

;WITH [set] AS
(
  SELECT TOP (50000) Number = ROW_NUMBER() 
    OVER (ORDER BY s1.[object_id])
	FROM sys.objects AS s1
	CROSS JOIN sys.objects AS s2
	ORDER BY Number
)
INSERT dbo.Numbers_Set WITH (TABLOCKX) (Number)
  SELECT Number FROM [set];

SELECT @Set = DATEDIFF(MILLISECOND, @d, SYSDATETIME());

-- set-based, using a Numbers table (we just created some):

SET @d = SYSDATETIME();

INSERT dbo.Numbers_Num WITH (TABLOCKX) (Number)
  SELECT Number FROM dbo.Numbers_Loop;

SELECT @Num = DATEDIFF(MILLISECOND, @d, SYSDATETIME());

SELECT 
  duration = 'milliseconds', 
  [loop] = @Loop, 
  [cte]  = @CTE, 
  [set]  = @Set,
  [num]  = @Num;
GO

DROP TABLE dbo.Numbers_Loop, 
           dbo.Numbers_CTE, 
		   dbo.Numbers_Set, 
		   dbo.Numbers_Num;