USE AdventureWorks2012;
GO

CREATE TABLE dbo.Calendar
(
  [date] DATE PRIMARY KEY,
  [weekday] TINYINT
)
WITH (DATA_COMPRESSION = PAGE);

-- get the days from 2000-01-01 => 2029-12-31:
;WITH [days](d) AS
(
  SELECT TOP (10958) DATEADD
  (
    DAY, 
    ROW_NUMBER() OVER (ORDER BY s1.[object_id])-1, 
	'20000101'
  )
  FROM sys.objects AS s1 CROSS JOIN sys.objects AS s2
)
INSERT dbo.Calendar WITH (TABLOCKX) ([date],[weekday])
  SELECT d, DATEPART(WEEKDAY, d) 
  FROM [days];

-- sample rows:
SELECT TOP (10) [date],[weekday] 
  FROM dbo.Calendar 
  ORDER BY [date];

-- 136 KB (200 KB if you turn off compression):
EXEC sp_spaceused 'dbo.Calendar';

-- sample usage:
DECLARE @d DATETIME = '20000101';

SELECT 
  TransactionDate = DATEADD(DAY,DATEDIFF(DAY,@d,TransactionDate),@d),
  [Units] = SUM(Quantity),
  [Cost]  = SUM(ActualCost),
  Transactions = COUNT(Quantity)
FROM
  Production.TransactionHistory
WHERE TransactionDate >= '20080701'
  AND TransactionDate <  '20080801'
  AND ProductID = 879
GROUP BY DATEADD(DAY,DATEDIFF(DAY,@d,TransactionDate),@d)
ORDER BY TransactionDate;

SELECT 
  c.[date], 
  [Units] = COALESCE(SUM(h.Quantity),0),
  [Cost]  = COALESCE(SUM(h.ActualCost),0), 
  [Transactions] = COUNT(h.Quantity)
FROM dbo.Calendar AS c
LEFT OUTER JOIN Production.TransactionHistory AS h
  ON    h.TransactionDate >= c.[date]
    AND h.TransactionDate < DATEADD(DAY, 1, c.[date])
	AND h.ProductID = 879
WHERE c.[date] >= '20080701'
  AND c.[date] <  '20080801'
GROUP BY c.[date]
ORDER BY c.[date];

-- clean up:
DROP TABLE dbo.Calendar;