SET NOCOUNT ON;

/*
   Shamelessly lifted from Itzik Ben-Gan
   http://sqlmag.com/t-sql/sql-server-read-consistency-problems-part-2 

   Demonstrates that NOLOCK can read the same row twice

   In this case, due to a page split moving a page already
   read in the current scan to an extent not yet encountered.

   For the same reason, it can skip rows, 

   And yes, this can happen under "safer" isolation levels too.
   Watch this developing series by Paul White:

   http://www.sqlperformance.com/2014/02/t-sql-queries/confusion-caused-by-trusting-acid
*/

CREATE DATABASE nolock_test;
GO
USE nolock_test;
GO

CREATE TABLE dbo.nolock_test
(
	id UNIQUEIDENTIFIER PRIMARY KEY DEFAULT(NEWID()),
	filler CHAR(2000) NOT NULL DEFAULT('x')
);

SET NOCOUNT ON;
GO
TRUNCATE TABLE dbo.nolock_test;

WHILE 1 = 1
	INSERT INTO dbo.nolock_test DEFAULT VALUES;
GO

/* in a separate connection! */

SET NOCOUNT ON;
USE nolock_test;
GO

WHILE 1 = 1
BEGIN
	SELECT * INTO #x FROM dbo.nolock_test WITH (NOLOCK);

	IF EXISTS
	(
      SELECT id FROM #x
		GROUP BY id
		HAVING COUNT(*) > 1
	)
	BEGIN
      SELECT id FROM #x
		GROUP BY id
		HAVING COUNT(*) > 1
	 BREAK;
    END
	DROP TABLE #x;
END

DROP TABLE #x;
GO


/* clean up */

USE master;
GO
ALTER DATABASE nolock_test SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
DROP DATABASE nolock_test;