-- Index Fillfactor for current database
SELECT
  DB_NAME() AS DatabaseName,
  SCHEMA_NAME(O.Schema_ID) AS SchemaName,
  OBJECT_NAME(s.[object_id]) AS TableName,
  i.name AS IndexName,
  i.fill_factor
FROM
  sys.dm_db_index_usage_stats s
  INNER JOIN sys.indexes i
    ON s.[object_id] = i.[object_id] AND
       s.index_id = i.index_id
  INNER JOIN sys.objects O
    ON i.object_id = O.object_id
WHERE
  s.database_id = DB_ID() AND
  i.name IS NOT NULL AND
  OBJECTPROPERTY(
    s.[object_id],
    'IsMsShipped') = 0
ORDER BY
  fill_factor DESC;
  
  -- DBCC SHOW_STATISTICS ([schema.tableName], indexName) WITH STAT_HEADER
  
-- Percentage of Fragmentation
SELECT i.name AS IndexName
, ROUND(avg_fragmentation_in_percent,2) AS [Fragmentation %]
FROM sys.dm_db_index_physical_stats(DB_ID('parisdev'),
OBJECT_ID('currency'), NULL, NULL, NULL) s
INNER JOIN sys.indexes i ON s.[object_id] = i.[object_id]
AND s.index_id = i.index_id;
  
-- Find tables that are missing indexes
SELECT TOP 20
ROUND(avg_total_user_cost * avg_user_impact 
* (user_seeks + user_scans),0) AS [Total Cost]
, [statement] AS [Table Name]
, equality_columns
, inequality_columns
, included_columns
FROM sys.dm_db_missing_index_groups g
INNER JOIN sys.dm_db_missing_index_group_stats s
ON s.group_handle = g.index_group_handle
INNER JOIN sys.dm_db_missing_index_details d
ON d.index_handle = g.index_handle
ORDER BY [Total Cost] DESC;


-- Find unused indexes
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT
DB_NAME() AS DatabaseName
, SCHEMA_NAME(O.Schema_ID) AS SchemaName
, OBJECT_NAME(s.[object_id]) AS TableName
, i.name AS IndexName
, user_updates
, system_seeks + system_scans + system_lookups AS [System usage]
INTO #TempUnusedIndexes
FROM sys.dm_db_index_usage_stats s
INNER JOIN sys.indexes i ON s.[object_id] = i.[object_id]
AND s.index_id = i.index_id
INNER JOIN sys.objects O ON i.object_id = O.object_id
WHERE 1=2;

EXEC sp_MSForEachDB 'USE [?]; 
INSERT INTO #TempUnusedIndexes
SELECT TOP 20
DB_NAME() AS DatabaseName
, SCHEMA_NAME(o.Schema_ID) AS SchemaName
, OBJECT_NAME(s.[object_id]) AS TableName
, i.name AS IndexName
, user_updates
, system_seeks + system_scans + system_lookups AS [System usage]
FROM sys.dm_db_index_usage_stats s
INNER JOIN sys.indexes i ON s.[object_id] = i.[object_id]
AND s.index_id = i.index_id
INNER JOIN sys.objects o ON i.object_id = O.object_id
WHERE s.database_id = DB_ID()
AND OBJECTPROPERTY(s.[object_id], ''IsMsShipped'') = 0
AND user_seeks = 0
AND user_scans = 0
AND user_lookups = 0
AND i.name IS NOT NULL
ORDER BY user_updates DESC';

SELECT TOP 20 * FROM #TempUnusedIndexes ORDER BY [user_updates] DESC;

DROP TABLE #TempUnusedIndexes;


-- Condition of index statistics?
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT
ss.name AS SchemaName
, st.name AS TableName
, s.name AS IndexName
, STATS_DATE(s.id,s.indid) AS 'Statistics Last Updated'
, s.rowcnt AS 'Row Count'
, s.rowmodctr AS 'Number Of Changes'
, CAST((CAST(s.rowmodctr AS DECIMAL(28,8))/CAST(s.rowcnt AS DECIMAL(28,2)) *
100.0)
AS DECIMAL(28,2)) AS '% Rows Changed'
FROM sys.sysindexes s
INNER JOIN sys.tables st ON st.[object_id] = s.[id]
INNER JOIN sys.schemas ss ON ss.[schema_id] = st.[schema_id]
WHERE s.id > 100
AND s.indid > 0
AND s.rowcnt >= 500
ORDER BY SchemaName, TableName, IndexName;


