--Quickly find the most used cached plans  simple version
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT TOP 10
st.text AS [SQL] 
, DB_NAME(st.dbid) AS DatabaseName
, cp.usecounts AS [Plan usage]
, qp.query_plan #2
FROM sys.dm_exec_cached_plans cp 
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) st
CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle) qp
WHERE st.text LIKE '%CREATE PROCEDURE%' 
ORDER BY cp.usecounts DESC;


-- Extracting the Individual Query from the parent batch
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT TOP 20
execution_count
, SUBSTRING (qt.text, (qs.statement_start_offset/2) + 1,
((CASE WHEN qs.statement_end_offset = -1
THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2
ELSE qs.statement_end_offset
END - qs.statement_start_offset)/2) + 1) AS [Individual Query]
, qt.text AS [Parent Query]
, DB_NAME(qt.dbid) AS DatabaseName
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) as qt
ORDER BY execution_count DESC;


-- Identify the database of ad-hoc queries and stored procedures
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT TOP 20
st.text AS [SQL]
,cp.cacheobjtype
,cp.objtype
, COALESCE(DB_NAME(st.dbid), 
DB_NAME(CAST(pa.value AS INT))+'*', 
'Resource') AS [DatabaseName] 
, cp.usecounts AS [Plan usage]
, qp.query_plan
FROM sys.dm_exec_cached_plans cp
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) st
CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle) qp
OUTER APPLY sys.dm_exec_plan_attributes(cp.plan_handle) pa
WHERE pa.attribute = 'dbid'
ORDER BY cp.usecounts DESC;


-- Determine query effect via differential between snapshots
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT sql_handle, plan_handle, total_elapsed_time 
, execution_count, statement_start_offset, statement_end_offset
INTO #PreWorkSnapShot
FROM sys.dm_exec_query_stats

--EXEC dbo.IWSR 

--SELECT * FROM dbo.appdate

SELECT sql_handle, plan_handle, total_elapsed_time 
, execution_count, statement_start_offset, statement_end_offset
INTO #PostWorkSnapShot
FROM sys.dm_exec_query_stats

SELECT
  p2.total_elapsed_time -
  ISNULL(
    p1.total_elapsed_time,
    0)
    AS [Duration],
  SUBSTRING(
    qt.text,
    p2.statement_start_offset / 2 + 1,
    ((CASE
        WHEN p2.statement_end_offset = -1 THEN LEN(CONVERT(
                                                     NVARCHAR(MAX),
                                                     qt.text)) *
                                               2
        ELSE p2.statement_end_offset
      END -
      p2.statement_start_offset) /
     2) +
    1)
    AS [Individual Query],
  qt.text AS [Parent Query],
  DB_NAME(qt.dbid) AS DatabaseName
FROM
  #PreWorkSnapShot p1
  RIGHT OUTER JOIN #PostWorkSnapShot p2
    ON p2.sql_handle = ISNULL(
                         p1.sql_handle,
                         p2.sql_handle) AND
       p2.plan_handle = ISNULL(
                          p1.plan_handle,
                          p2.plan_handle) AND
       p2.statement_start_offset = ISNULL(
                                     p1.statement_start_offset,
                                     p2.statement_start_offset) AND
       p2.statement_end_offset = ISNULL(
                                   p1.statement_end_offset,
                                   p2.statement_end_offset)
  CROSS APPLY sys.dm_exec_sql_text(p2.sql_handle) AS qt
WHERE
  p2.execution_count != ISNULL(
                          p1.execution_count,
                          0)
ORDER BY
  [Duration] DESC

DROP TABLE #PreWorkSnapShot

DROP TABLE #PostWorkSnapShot;