-- DMV: What are the active transactions for a single database?

	SELECT * FROM sys.dm_tran_database_transactions 
	WHERE database_id = 8

-- DMF: Whats the IO for my databases and files?

	SELECT * FROM sys.dm_io_virtual_file_stats(NULL,NULL) 
	ORDER BY database_id, file_id;
  

-- Lets collect some vital stats on how busy database files have been since the last restart:

SELECT
  DB_NAME(mf.database_id) AS databaseName,
  mf.physical_name,
  num_of_reads,
  num_of_bytes_read,
  io_stall_read_ms,
  num_of_writes,
  num_of_bytes_written,
  io_stall_write_ms,
  io_stall,
  size_on_disk_bytes
FROM
    sys.dm_io_virtual_file_stats(
      NULL,
      NULL) AS divfs
  JOIN
    sys.master_files AS mf  ON mf.database_id = divfs.database_id AND mf.file_id = divfs.file_id;
    
--Pages used and total pages for a heap and clustered index:

SELECT object_id,
       SUM(in_row_data_page_count) AS DataPageCount,
       SUM(in_row_used_page_count) AS UsedPageCount
FROM sys.dm_db_partition_stats 
WHERE index_id = 0 OR index_id = 1 -- heap or clustered index
GROUP BY object_id
ORDER BY object_id;


--- DMV reports statements with lowest plan reuse
---
SELECT   TOP 10
         qs.sql_handle,
         qs.plan_handle,
         cp.cacheobjtype,
         cp.usecounts,
         cp.size_in_bytes,
         qs.statement_start_offset,
         qs.statement_end_offset,
         qt.dbid,
         qt.objectid,
         qt.text,
         SUBSTRING(qt.text, qs.statement_start_offset / 2, 
            (CASE 
                WHEN qs.statement_end_offset = -1 
                THEN len(CONVERT(NVARCHAR(MAX), qt.text)) * 2 
                ELSE qs.statement_end_offset 
             END - qs.statement_start_offset) / 2) AS statement
FROM     sys.dm_exec_query_stats qs
         CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS qt
         INNER JOIN sys.dm_exec_cached_plans AS cp
           ON qs.plan_handle = cp.plan_handle
WHERE cp.plan_handle = qs.plan_handle 
  AND qt.dbid = db_id() ----- put the database ID here
ORDER BY [Usecounts] ASC

SELECT *
FROM sys.dm_exec_query_stats