--Get plan handles
SELECT TOP 20 qt.text, qs.last_execution_time
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) as qt;

-- Find your slowest queries
SELECT
  TOP 20
  CAST(total_elapsed_time / 1000000.0 AS DECIMAL(28, 2))
    AS [Total Elapsed Duration (s)],
  execution_count,
  SUBSTRING(
    qt.text,
    (qs.statement_start_offset / 2) + 1,
    ((CASE
        WHEN qs.statement_end_offset = -1 THEN LEN(CONVERT(NVARCHAR(MAX),qt.text)) * 2
        ELSE qs.statement_end_offset
      END -
      qs.statement_start_offset) / 2) + 1)
    AS [Individual Query],
  qt.text AS [Parent Query],
  DB_NAME(qt.dbid) AS DatabaseName,
  qp.query_plan
FROM
  sys.dm_exec_query_stats qs
  CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS qt
  CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) qp
  INNER JOIN sys.dm_exec_cached_plans AS cp
    ON qs.plan_handle = cp.plan_handle
ORDER BY
  total_elapsed_time DESC;
  
  
  
-- Find Missing Indexes
SELECT TOP 20
ROUND(avg_total_user_cost * avg_user_impact * (user_seeks + user_scans),0) AS [Total Cost]
, avg_user_impact
, statement AS TableName
, equality_columns
, inequality_columns
, included_columns
FROM sys.dm_db_missing_index_groups g
INNER JOIN sys.dm_db_missing_index_group_stats s
ON s.group_handle = g.index_group_handle
INNER JOIN sys.dm_db_missing_index_details d
ON d.index_handle = g.index_handle
ORDER BY [Total Cost] DESC;


-- What SQL is running now?
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

SELECT session_Id AS [Spid]
, ecid
, DB_NAME(sp.dbid) AS [Database]
, nt_username
, er.status
, wait_type
, SUBSTRING (qt.text, (er.statement_start_offset/2) + 1, 
((CASE WHEN er.statement_end_offset = -1
THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2
ELSE er.statement_end_offset
END - er.statement_start_offset)/2) + 1) AS [Individual Query]
, qt.text AS [Parent Query]
, program_name
, Hostname
, nt_domain
, start_time
FROM sys.dm_exec_requests er 
INNER JOIN sys.sysprocesses sp ON er.session_id = sp.spid
CROSS APPLY sys.dm_exec_sql_text(er.sql_handle)as qt
WHERE session_Id > 50
AND session_Id NOT IN (@@SPID)
ORDER BY session_Id, ecid;

-- List of cached plans
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT TOP 20
st.text AS [SQL]
, cp.cacheobjtype
, cp.objtype
, COALESCE(DB_NAME(st.dbid),
    DB_NAME(CAST(pa.value AS INT))+'*', 'Resource') AS [DatabaseName]
, cp.usecounts AS [Plan usage]
, qp.query_plan
FROM sys.dm_exec_cached_plans cp 
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) st
CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle) qp
OUTER APPLY sys.dm_exec_plan_attributes(cp.plan_handle) pa
WHERE pa.attribute = 'dbid'
AND st.text LIKE '%CREATE PROCEDURE%';


-- Cached plans?
SELECT TOP 20
st.text AS [SQL]
, cp.cacheobjtype
, cp.objtype
, COALESCE(DB_NAME(st.dbid),
DB_NAME(CAST(pa.value AS INT))+'*',
'Resource') AS [DatabaseName]
, cp.usecounts AS [Plan usage]
, qp.query_plan
FROM sys.dm_exec_cached_plans cp 
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) st
CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle) qp
OUTER APPLY sys.dm_exec_plan_attributes(cp.plan_handle) pa
WHERE pa.attribute = 'dbid'
AND st.text LIKE '%PartyType%';


-- The queries spend the longest time being blocked
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT TOP 20
CAST((total_elapsed_time - total_worker_time) / 1000000.0 
AS DECIMAL(28,2)) AS [Total time blocked (s)]
, CAST(total_worker_time * 100.0 / total_elapsed_time
AS DECIMAL(28,2)) AS [% CPU]
, CAST((total_elapsed_time - total_worker_time)* 100.0 /
total_elapsed_time AS DECIMAL(28, 2)) AS [% Waiting]
, execution_count
, CAST((total_elapsed_time - total_worker_time) / 1000000.0
/ execution_count AS DECIMAL(28, 2)) AS [Blocking average (s)]
, SUBSTRING (qt.text,(qs.statement_start_offset/2) + 1, 
((CASE WHEN qs.statement_end_offset = -1
THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2
ELSE qs.statement_end_offset
END - qs.statement_start_offset)/2) + 1) AS [Individual Query]
, qt.text AS [Parent Query]
, DB_NAME(qt.dbid) AS DatabaseName
, qp.query_plan
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) as qt
CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) qp
INNER JOIN sys.dm_exec_cached_plans as cp
on qs.plan_handle=cp.plan_handle
WHERE total_elapsed_time > 0
ORDER BY [Total time blocked (s)] DESC ;


--The queries that have been executed the most often
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT TOP 20
execution_count
, SUBSTRING (qt.text,(qs.statement_start_offset/2) + 1, 
((CASE WHEN qs.statement_end_offset = -1
THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2
ELSE qs.statement_end_offset
END - qs.statement_start_offset)/2) + 1) AS [Individual Query]
, qt.text AS [Parent Query]
, DB_NAME(qt.dbid) AS DatabaseName
, qp.query_plan
FROM sys.dm_exec_query_stats qs
CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) as qt
CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) qp
ORDER BY execution_count DESC;